	! file: gauss.f
	! function: gauss-jordan elimination program with result check,
	!	    operating on a straight a matrix. (every row contains
	!    	    the coefficients of an equation)
	! behaviour: number of mflops decreases with increasing dimension,
	!	    and amount of data communicated between threads increases
	!	    proportional to n^3.
	! author: BVA, 21.08.96.
	parameter(nmax=1023)
	dimension a(nmax+1,nmax), x(nmax) 
	dimension b(nmax+1,nmax), c(nmax) 
	! static a, b, c, x
	real a, b, c, x
	double precision it0,it1,it2,it3
	real mflops,fp_ops,delta
 	character*20 argv
	
        ! Initialize n
	narg=iargc()
	if (narg .gt. 1) then
	   call getarg(1,argv)
	   read(argv,*) n
	else
	   n = 4
	endif
        ! Initialize a(,)
	!$dosingle
C$DOISV
	do i=1, n
	   a(i,n+1) = 1.0
	   !$dosingle
	   do j=1, n
	      if (i.eq.j) then
		 a(i,j) = i
	      else
		 a(i,j) = 3.0*sin(dble(i+j))
	      endif
	   enddo
	enddo
        ! Initialize b(,)
	!$dosingle
	do i=1,n
	   !$dosingle
	   do j=1,n+1
	      b(i,j)=a(i,j)
	   enddo
	enddo
	! subtract f times column i from column j, with i and j from 1 to n
	do i = 1,n  
	   !$par doall
	   do j = 1,n
	      if(j.ne.i) then
	         !write (*,*) "1 w f"
	         !write (*,*) "1 r a",j,i
	         !write (*,*) "1 r a",i,i
		 f=a(j,i)/a(i,i)
		 do k=i+1,n+1
		    !write (*,*) "2 w a",j,k
	            !write (*,*) "2 r f"
	            !write (*,*) "2 r a",j,k
	            !write (*,*) "2 r a",i,k
		    a(j,k)=a(j,k)-f*a(i,k)
		 enddo
	      endif
	   enddo
	enddo
	it2=it2-it1
	! substitution
	!$dosingle
	do i=1,n
	   x(i)=a(i,n+1)/a(i,i)
	enddo
	if (n.eq.10) write(*,"(1x, 10f8.3/)") (x(i), i=1, n)
	! backsubstitution
	!$dosingle
	do i=1,n
	   c(i) = 0
	   !$dosingle
	   do j=1,n
	      c(i) = c(i) + b(i,j)*x(j)
	   enddo
	   delta = delta + (c(i)-b(i,n+1))**2
	enddo
	it3=it3-it0
	fp_ops = (n-1)*n*(n+2)
	mflops = fp_ops/(it2 * 1e+6)
	write(*,*) "size: ", n, ", average error: ", sqrt(delta)/n
	write(*,*) "total: ", it3, ", main loop: ", it2
	write(*,*) "flops: ", fp_ops, ", mflops: ", mflops 
	end
